import 'dart:convert';
import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';

class SoopulaeTexView extends StatefulWidget {
  final String tex;
  final Color backgroundColor;
  final Color captureBackgroundColor;
  final VoidCallback? onRenderFinished;
  final void Function(Uint8List pngData, int width, int height)?
      onCaptureFinished;

  const SoopulaeTexView(
      {Key? key,
      required this.tex,
      this.backgroundColor = Colors.transparent,
      this.captureBackgroundColor = Colors.transparent,
      this.onRenderFinished,
      this.onCaptureFinished})
      : super(key: key);

  @override
  State<SoopulaeTexView> createState() => _SoopulaeTexViewState();
}

class _SoopulaeTexViewState extends State<SoopulaeTexView> {
  late final InAppWebViewController _webViewController;

  double _height = 1;

  String getCssColor(Color color) {
    return "rgba(${widget.backgroundColor.red}, ${widget.backgroundColor.green}, ${widget.backgroundColor.blue}, ${widget.backgroundColor.alpha})";
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: _height,
      child: InAppWebView(
        initialFile: "packages/soopulae_tex/assets/index.html",
        onWebViewCreated: (controller) {
          _webViewController = controller;

          _webViewController.addJavaScriptHandler(
            handlerName: "requestInit",
            callback: (_) {
              return [
                widget.tex,
                getCssColor(widget.backgroundColor),
                getCssColor(widget.captureBackgroundColor)
              ];
            },
          );
          _webViewController.addJavaScriptHandler(
            handlerName: "katexRendered",
            callback: (args) {
              setState(() {
                _height = (args[1] as num).toDouble() + 10 /* 아래 부분이 잘리는 현상 막기 위해 높이 추가 */;
              });

              if (widget.onRenderFinished != null) {
                widget.onRenderFinished!();
              }
            },
          );
          _webViewController.addJavaScriptHandler(
            handlerName: "captureFinished",
            callback: (args) {
              final String base64png = args[0] as String;
              final int width = args[1] as int;
              final int height = args[2] as int;

              final Uint8List pngData = base64Decode(base64png.split(",")[1]);

              if (widget.onCaptureFinished != null) {
                widget.onCaptureFinished!(pngData, width, height);
              }
            },
          );
        },
        onConsoleMessage: (_, message) {
          print("console message : ${message.message}");
        },
      ),
    );
  }
}
